#pragma once

// Genode includes
#include <base/exception.h>
#include <util/string.h>

namespace Qumela {
	template <typename T, Genode::size_t N>
	class Quantity;

	class Unexpected_content : public Genode::Exception { };

	using Genode::Output;
};


/**
 * Wrapper for a value that has an unit.
 */
template <typename T, Genode::size_t N>
class Qumela::Quantity
{
	public:

		typedef Genode::String<N+1> String;

	private:

		T      _value;
		String _unit;

	public:

		/**
		 * Default constructor
		 */
		Quantity() noexcept : _value{0}, _unit{ } { }

		/**
		 * Constructor, to be used implicitly via assignment operator
		 */
		Quantity(T value, String unit) :
		        _value{value}, _unit{unit}
		{ }

		inline T      value() const noexcept { return _value; }
		inline String unit()  const          { return _unit; }

		void print(Output &output) const
		{
			using Genode::print;

			print(output, _value);
			print(output, _unit);
		}
};


namespace Genode {

	/**
	 * Read a value together with its unit.
	 *
	 * \return number of consumed characters
	 */
	template <typename T, Genode::size_t N>
	inline size_t ascii_to(const char *s, Qumela::Quantity<T, N> &result)
	{
		T      value{0};
		size_t i = ascii_to(s, value);
		char   buf[N];
		memset(buf, 0, sizeof(buf));
		for (size_t n=0; (n<N) && (s[i+n] != '\0') && !end_of_quote(&s[i+n-1]); ++n)
			buf[n] = s[i+n];
		result = {value, buf};
		return i + result.unit().length();
	}

};


/**
 * The following include needs to be after the definitions above, so that the
 * XML functions can find the ascii_to() for the Qumela types.
 */
#include <util/xml_node.h>


namespace Qumela {

	using Genode::Xml_node;
	using namespace Genode;

	/**
	 * Return content of a node with default value
	 *
	 * \param default_value  default value
	 * \return               XML content
	 *
	 * In case of an error while parsing the content, the default
	 * value given will be returned.
	 */
	template <typename T>
	inline T xml_node_content_value(const Genode::Xml_node& node, T default_value)
	{
		T result = default_value;
		T temp_result = T();
		if (ascii_to(node.content_base(), temp_result) == node.content_size()) {
			result = temp_result;
		}
		return result;
	}

	/**
	 * Return content of a node
	 *
	 * \throw Unexpected_content  unexpected content
	 * \return                    XML content
	 */
	template <typename T>
	inline T xml_node_content_value(const Genode::Xml_node& node)
	{
		T result = T();
		if (ascii_to(node.content_base(), result) != node.content_size()) {
			throw Unexpected_content();
		}
		return result;
	}

}
